@@ -526,6 +526,14 @@ func (c *Client) DeleteService(svc *v1.Service) error {
 	return err
 }
 
+func (c *Client) DeleteRoute(r *routev1.Route) error {
+	err := c.osrclient.RouteV1().Routes(r.GetNamespace()).Delete(r.GetName(), &metav1.DeleteOptions{})
+	if apierrors.IsNotFound(err) {
+		return nil
+	}
+	return err
+}
+
 func (c *Client) DeletePrometheusRule(rule *monv1.PrometheusRule) error {
 	err := c.mclient.MonitoringV1().PrometheusRules(rule.GetNamespace()).Delete(rule.GetName(), &metav1.DeleteOptions{})
 	if apierrors.IsNotFound(err) {
@@ -543,6 +551,14 @@ func (c *Client) DeleteSecret(s *v1.Secret) error {
 	return err
 }
 
+func (c *Client) DeleteValidatingWebhookConfiguration(name string) error {
+	err := c.kclient.AdmissionregistrationV1().ValidatingWebhookConfigurations().Delete(name, &metav1.DeleteOptions{})
+	if apierrors.IsNotFound(err) {
+		return nil
+	}
+	return err
+}
+
 func (c *Client) WaitForPrometheus(p *monv1.Prometheus) error {
 	var lastErr error
 	if err := wait.Poll(time.Second*10, time.Minute*5, func() (bool, error) {
