@@ -46,6 +46,8 @@ type Config struct {
 	TelemeterClientConfig  *TelemeterClientConfig       `json:"telemeterClient"`
 	K8sPrometheusAdapter   *K8sPrometheusAdapter        `json:"k8sPrometheusAdapter"`
 	UserWorkloadConfig     *UserWorkloadConfig          `json:"techPreviewUserWorkload"`
+	ThanosRulerConfig      *ThanosRulerConfig           `json:"thanosRuler"`
+	ThanosQuerierConfig    *ThanosQuerierConfig         `json:"thanosQuerier"`
 }
 
 type Images struct {
@@ -97,6 +99,19 @@ type AlertmanagerMainConfig struct {
 	Hostport            string                    `json:"hostport"`
 }
 
+type ThanosRulerConfig struct {
+	NodeSelector        map[string]string         `json:"nodeSelector"`
+	Tolerations         []v1.Toleration           `json:"tolerations"`
+	Resources           *v1.ResourceRequirements  `json:"resources"`
+	VolumeClaimTemplate *v1.PersistentVolumeClaim `json:"volumeClaimTemplate"`
+}
+
+type ThanosQuerierConfig struct {
+	NodeSelector map[string]string        `json:"nodeSelector"`
+	Tolerations  []v1.Toleration          `json:"tolerations"`
+	Resources    *v1.ResourceRequirements `json:"resources"`
+}
+
 type GrafanaConfig struct {
 	NodeSelector map[string]string `json:"nodeSelector"`
 	Tolerations  []v1.Toleration   `json:"tolerations"`
@@ -206,6 +221,12 @@ func (c *Config) applyDefaults() {
 	if c.AlertmanagerMainConfig == nil {
 		c.AlertmanagerMainConfig = &AlertmanagerMainConfig{}
 	}
+	if c.ThanosRulerConfig == nil {
+		c.ThanosRulerConfig = &ThanosRulerConfig{}
+	}
+	if c.ThanosQuerierConfig == nil {
+		c.ThanosQuerierConfig = &ThanosQuerierConfig{}
+	}
 	if c.GrafanaConfig == nil {
 		c.GrafanaConfig = &GrafanaConfig{}
 	}
