@@ -58,6 +58,7 @@ var (
 	AlertmanagerServiceAccount     = "assets/alertmanager/service-account.yaml"
 	AlertmanagerClusterRoleBinding = "assets/alertmanager/cluster-role-binding.yaml"
 	AlertmanagerClusterRole        = "assets/alertmanager/cluster-role.yaml"
+	AlertmanagerRBACProxySecret    = "assets/alertmanager/kube-rbac-proxy-secret.yaml"
 	AlertmanagerRoute              = "assets/alertmanager/route.yaml"
 	AlertmanagerServiceMonitor     = "assets/alertmanager/service-monitor.yaml"
 	AlertmanagerTrustedCABundle    = "assets/alertmanager/trusted-ca-bundle.yaml"
@@ -164,6 +165,9 @@ var (
 	ClusterMonitoringOperatorService        = "assets/cluster-monitoring-operator/service.yaml"
 	ClusterMonitoringOperatorServiceMonitor = "assets/cluster-monitoring-operator/service-monitor.yaml"
 	ClusterMonitoringClusterRole            = "assets/cluster-monitoring-operator/cluster-role.yaml"
+	ClusterMonitoringRulesEditClusterRole   = "assets/cluster-monitoring-operator/monitoring-rules-edit-cluster-role.yaml"
+	ClusterMonitoringRulesViewClusterRole   = "assets/cluster-monitoring-operator/monitoring-rules-view-cluster-role.yaml"
+	ClusterMonitoringEditClusterRole        = "assets/cluster-monitoring-operator/monitoring-edit-cluster-role.yaml"
 	ClusterMonitoringGrpcTLSSecret          = "assets/cluster-monitoring-operator/grpc-tls-secret.yaml"
 
 	TelemeterClientClusterRole            = "assets/telemeter-client/cluster-role.yaml"
@@ -188,6 +192,22 @@ var (
 	ThanosQuerierGrpcTLSSecret      = "assets/thanos-querier/grpc-tls-secret.yaml"
 	ThanosQuerierTrustedCABundle    = "assets/thanos-querier/trusted-ca-bundle.yaml"
 
+	ThanosRulerCustomResource               = "assets/thanos-ruler/thanos-ruler.yaml"
+	ThanosRulerService                      = "assets/thanos-ruler/service.yaml"
+	ThanosRulerRoute                        = "assets/thanos-ruler/route.yaml"
+	ThanosRulerOauthCookieSecret            = "assets/thanos-ruler/oauth-cookie-secret.yaml"
+	ThanosRulerHtpasswdSecret               = "assets/thanos-ruler/oauth-htpasswd-secret.yaml"
+	ThanosRulerQueryConfigSecret            = "assets/thanos-ruler/query-config-secret.yaml"
+	ThanosRulerAlertmanagerConfigSecret     = "assets/thanos-ruler/alertmanagers-config-secret.yaml"
+	ThanosRulerServiceAccount               = "assets/thanos-ruler/service-account.yaml"
+	ThanosRulerClusterRole                  = "assets/thanos-ruler/cluster-role.yaml"
+	ThanosRulerClusterRoleBinding           = "assets/thanos-ruler/cluster-role-binding.yaml"
+	ThanosRulerMonitoringClusterRoleBinding = "assets/thanos-ruler/cluster-role-binding-monitoring.yaml"
+	ThanosRulerGrpcTLSSecret                = "assets/thanos-ruler/grpc-tls-secret.yaml"
+	ThanosRulerTrustedCABundle              = "assets/thanos-ruler/trusted-ca-bundle.yaml"
+	ThanosRulerServiceMonitor               = "assets/thanos-ruler/service-monitor.yaml"
+	ThanosRulerPrometheusRule               = "assets/thanos-ruler/thanos-ruler-prometheus-rule.yaml"
+
 	TelemeterTrustedCABundle = "assets/telemeter-client/trusted-ca-bundle.yaml"
 )
 
@@ -203,8 +223,7 @@ var (
 	AuthProxyCookieDomainFlag = "-cookie-domain="
 	AuthProxyRedirectURLFlag  = "-redirect-url="
 
-	PrometheusTrustedCABundleDir  = "/etc/pki/prometheus-ca-bundle/"
-	PrometheusTrustedCABundlePath = PrometheusTrustedCABundleDir + "ca-bundle.crt"
+	TrustedCABundleKey = "ca-bundle.crt"
 )
 
 const (
@@ -369,6 +388,7 @@ func (f *Factory) AlertmanagerMain(host string, trustedCABundleCM *v1.ConfigMap)
 		for i := range container.Env {
 			if container.Env[i].Name == name {
 				container.Env[i].Value = value
+				break
 			}
 		}
 	}
@@ -376,6 +396,7 @@ func (f *Factory) AlertmanagerMain(host string, trustedCABundleCM *v1.ConfigMap)
 		switch c.Name {
 		case "alertmanager-proxy":
 			a.Spec.Containers[i].Image = f.config.Images.OauthProxy
+
 			if f.config.HTTPConfig.HTTPProxy != "" {
 				setEnv(&a.Spec.Containers[i], "HTTP_PROXY", f.config.HTTPConfig.HTTPProxy)
 			}
@@ -391,7 +412,7 @@ func (f *Factory) AlertmanagerMain(host string, trustedCABundleCM *v1.ConfigMap)
 				a.Spec.VolumeMounts = append(a.Spec.VolumeMounts, trustedCABundleVolumeMount(volumeName))
 				volume := trustedCABundleVolume(trustedCABundleCM.Name, volumeName)
 				volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
-					Key:  "ca-bundle.crt",
+					Key:  TrustedCABundleKey,
 					Path: "tls-ca-bundle.pem",
 				})
 				a.Spec.Volumes = append(a.Spec.Volumes, volume)
@@ -400,6 +421,10 @@ func (f *Factory) AlertmanagerMain(host string, trustedCABundleCM *v1.ConfigMap)
 					trustedCABundleVolumeMount(volumeName),
 				)
 			}
+		case "kube-rbac-proxy":
+			a.Spec.Containers[i].Image = f.config.Images.KubeRbacProxy
+		case "prom-label-proxy":
+			a.Spec.Containers[i].Image = f.config.Images.PromLabelProxy
 		}
 	}
 
@@ -408,6 +433,17 @@ func (f *Factory) AlertmanagerMain(host string, trustedCABundleCM *v1.ConfigMap)
 	return a, nil
 }
 
+func (f *Factory) AlertmanagerRBACProxySecret() (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(AlertmanagerRBACProxySecret))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespace
+
+	return s, nil
+}
+
 func (f *Factory) AlertmanagerRoute() (*routev1.Route, error) {
 	r, err := f.NewRoute(MustAssetReader(AlertmanagerRoute))
 	if err != nil {
@@ -455,10 +491,14 @@ func (f *Factory) KubeStateMetricsDeployment() (*appsv1.Deployment, error) {
 	if err != nil {
 		return nil, err
 	}
-
-	d.Spec.Template.Spec.Containers[0].Image = f.config.Images.KubeRbacProxy
-	d.Spec.Template.Spec.Containers[1].Image = f.config.Images.KubeRbacProxy
-	d.Spec.Template.Spec.Containers[2].Image = f.config.Images.KubeStateMetrics
+	for i, container := range d.Spec.Template.Spec.Containers {
+		if container.Name == "kube-state-metrics" {
+			d.Spec.Template.Spec.Containers[i].Image = f.config.Images.KubeStateMetrics
+		}
+		if container.Name == "kube-rbac-proxy-self" || container.Name == "kube-rbac-proxy-main" {
+			d.Spec.Template.Spec.Containers[i].Image = f.config.Images.KubeRbacProxy
+		}
+	}
 
 	if f.config.KubeStateMetricsConfig.NodeSelector != nil {
 		d.Spec.Template.Spec.NodeSelector = f.config.KubeStateMetricsConfig.NodeSelector
@@ -934,6 +974,17 @@ func (f *Factory) ThanosQuerierHtpasswdSecret(password string) (*v1.Secret, erro
 	return s, nil
 }
 
+func (f *Factory) ThanosRulerHtpasswdSecret(password string) (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(ThanosRulerHtpasswdSecret))
+	if err != nil {
+		return nil, err
+	}
+
+	f.generateHtpasswdSecret(s, password)
+	s.Namespace = f.namespaceUserWorkload
+	return s, nil
+}
+
 func (f *Factory) generateHtpasswdSecret(s *v1.Secret, password string) {
 	// #nosec
 	// TODO: Replace this with a safer algorithm
@@ -943,6 +994,26 @@ func (f *Factory) generateHtpasswdSecret(s *v1.Secret, password string) {
 	s.Namespace = f.namespace
 }
 
+func (f *Factory) ThanosRulerQueryConfigSecret() (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(ThanosRulerQueryConfigSecret))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespaceUserWorkload
+	return s, nil
+}
+
+func (f *Factory) ThanosRulerAlertmanagerConfigSecret() (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(ThanosRulerAlertmanagerConfigSecret))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespaceUserWorkload
+	return s, nil
+}
+
 func (f *Factory) PrometheusRBACProxySecret() (*v1.Secret, error) {
 	s, err := f.NewSecret(MustAssetReader(PrometheusRBACProxySecret))
 	if err != nil {
@@ -1024,7 +1095,7 @@ func (f *Factory) PrometheusK8sEtcdSecret(tlsClient *v1.Secret, ca *v1.ConfigMap
 	r := newErrMapReader(data)
 
 	var (
-		clientCA   = r.value("ca-bundle.crt")
+		clientCA   = r.value(TrustedCABundleKey)
 		clientCert = r.value("tls.crt")
 		clientKey  = r.value("tls.key")
 	)
@@ -1178,15 +1249,24 @@ func (f *Factory) PrometheusK8s(host string, grpcTLS *v1.Secret, trustedCABundle
 			URL:         f.config.TelemeterClientConfig.TelemeterServerURL,
 			BearerToken: base64.StdEncoding.EncodeToString(compositeToken),
 			QueueConfig: &monv1.QueueConfig{
-				Capacity: 10000,
-				// Default: 100
-				MaxSamplesPerSend: 1000,
-				// Default: 1000
-				MaxShards: 100,
-				// Default: 30ms
-				MinBackoff: "500ms",
-				// Default: 100ms
-				MaxBackoff: "2s",
+				// Amount of samples to load from the WAL into the in-memory
+				// buffer before waiting for samples to be sent successfully
+				// and then continuing to read from the WAL.
+				Capacity: 30000,
+				// Should we accumulate 10000 samples before the batch send
+				// deadline is reached, we will send this amount of samples
+				// anyways.
+				MaxSamplesPerSend: 10000,
+				// Batch samples for 1m until we send them if we not reach the
+				// 10000 MaxSamplesPerSend first.
+				BatchSendDeadline: "1m",
+				// Backoff is doubled on every backoff. We start with 1s
+				// backoff and double until the MaxBackOff.
+				MinBackoff: "1s",
+				// 128s is the 8th backoff in a row, once we end up here, we
+				// don't increase backoff time anymore. As we would at most
+				// produce (concurrency/256) number of requests per second.
+				MaxBackoff: "256s",
 			},
 			WriteRelabelConfigs: []monv1.RelabelConfig{
 				*selectorRelabelConfig,
@@ -1280,7 +1360,7 @@ func (f *Factory) PrometheusK8s(host string, grpcTLS *v1.Secret, trustedCABundle
 		volumeName := "prometheus-trusted-ca-bundle"
 		volume := trustedCABundleVolume(trustedCABundleCM.Name, volumeName)
 		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
-			Key:  "ca-bundle.crt",
+			Key:  TrustedCABundleKey,
 			Path: "tls-ca-bundle.pem",
 		})
 		p.Spec.Volumes = append(p.Spec.Volumes, volume)
@@ -1620,6 +1700,7 @@ func (f *Factory) PrometheusOperatorServiceMonitor() (*monv1.ServiceMonitor, err
 	}
 
 	sm.Namespace = f.namespace
+	sm.Spec.Endpoints[0].TLSConfig.ServerName = fmt.Sprintf("prometheus-operator.%s.svc", f.namespace)
 
 	return sm, nil
 }
@@ -1631,6 +1712,7 @@ func (f *Factory) PrometheusOperatorUserWorkloadServiceMonitor() (*monv1.Service
 	}
 
 	sm.Namespace = f.namespaceUserWorkload
+	sm.Spec.Endpoints[0].TLSConfig.ServerName = fmt.Sprintf("prometheus-operator.%s.svc", f.namespaceUserWorkload)
 
 	return sm, nil
 }
@@ -1702,26 +1784,27 @@ func (f *Factory) PrometheusOperatorDeployment(namespaces []string) (*appsv1.Dep
 	}
 
 	d.Spec.Template.Spec.Containers[0].Image = f.config.Images.PrometheusOperator
+	d.Spec.Template.Spec.Containers[1].Image = f.config.Images.KubeRbacProxy
 
 	args := d.Spec.Template.Spec.Containers[0].Args
 	for i := range args {
-		if strings.HasPrefix(args[i], PrometheusOperatorNamespaceFlag) {
+		if strings.HasPrefix(args[i], PrometheusOperatorNamespaceFlag) && len(namespaces) > 0 {
 			args[i] = PrometheusOperatorNamespaceFlag + strings.Join(namespaces, ",")
 		}
 
-		if strings.HasPrefix(args[i], PrometheusConfigReloaderFlag) {
+		if strings.HasPrefix(args[i], PrometheusConfigReloaderFlag) && f.config.Images.PrometheusConfigReloader != "" {
 			args[i] = PrometheusConfigReloaderFlag + f.config.Images.PrometheusConfigReloader
 		}
 
-		if strings.HasPrefix(args[i], ConfigReloaderImageFlag) {
+		if strings.HasPrefix(args[i], ConfigReloaderImageFlag) && f.config.Images.ConfigmapReloader != "" {
 			args[i] = ConfigReloaderImageFlag + f.config.Images.ConfigmapReloader
 		}
 
-		if strings.HasPrefix(args[i], PrometheusOperatorAlertmanagerInstanceNamespacesFlag) {
+		if strings.HasPrefix(args[i], PrometheusOperatorAlertmanagerInstanceNamespacesFlag) && f.namespace != "" {
 			args[i] = PrometheusOperatorAlertmanagerInstanceNamespacesFlag + f.namespace
 		}
 
-		if strings.HasPrefix(args[i], PrometheusOperatorPrometheusInstanceNamespacesFlag) {
+		if strings.HasPrefix(args[i], PrometheusOperatorPrometheusInstanceNamespacesFlag) && f.namespace != "" {
 			args[i] = PrometheusOperatorPrometheusInstanceNamespacesFlag + f.namespace
 		}
 	}
@@ -1746,6 +1829,7 @@ func (f *Factory) PrometheusOperatorUserWorkloadDeployment(denyNamespaces []stri
 	}
 
 	d.Spec.Template.Spec.Containers[0].Image = f.config.Images.PrometheusOperator
+	d.Spec.Template.Spec.Containers[1].Image = f.config.Images.KubeRbacProxy
 
 	args := d.Spec.Template.Spec.Containers[0].Args
 	for i := range args {
@@ -1999,7 +2083,7 @@ func (f *Factory) GrafanaDeployment(proxyCABundleCM *v1.ConfigMap) (*appsv1.Depl
 		d.Spec.Template.Spec.Containers[1].VolumeMounts = append(d.Spec.Template.Spec.Containers[1].VolumeMounts, trustedCABundleVolumeMount(volumeName))
 		volume := trustedCABundleVolume(proxyCABundleCM.Name, volumeName)
 		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
-			Key:  "ca-bundle.crt",
+			Key:  TrustedCABundleKey,
 			Path: "tls-ca-bundle.pem",
 		})
 		d.Spec.Template.Spec.Volumes = append(d.Spec.Template.Spec.Volumes, volume)
@@ -2083,6 +2167,33 @@ func (f *Factory) ClusterMonitoringClusterRole() (*rbacv1.ClusterRole, error) {
 	return cr, nil
 }
 
+func (f *Factory) ClusterMonitoringRulesEditClusterRole() (*rbacv1.ClusterRole, error) {
+	cr, err := f.NewClusterRole(MustAssetReader(ClusterMonitoringRulesEditClusterRole))
+	if err != nil {
+		return nil, err
+	}
+
+	return cr, nil
+}
+
+func (f *Factory) ClusterMonitoringRulesViewClusterRole() (*rbacv1.ClusterRole, error) {
+	cr, err := f.NewClusterRole(MustAssetReader(ClusterMonitoringRulesViewClusterRole))
+	if err != nil {
+		return nil, err
+	}
+
+	return cr, nil
+}
+
+func (f *Factory) ClusterMonitoringEditClusterRole() (*rbacv1.ClusterRole, error) {
+	cr, err := f.NewClusterRole(MustAssetReader(ClusterMonitoringEditClusterRole))
+	if err != nil {
+		return nil, err
+	}
+
+	return cr, nil
+}
+
 func (f *Factory) ClusterMonitoringOperatorService() (*v1.Service, error) {
 	s, err := f.NewService(MustAssetReader(ClusterMonitoringOperatorService))
 	if err != nil {
@@ -2100,6 +2211,7 @@ func (f *Factory) ClusterMonitoringOperatorServiceMonitor() (*monv1.ServiceMonit
 		return nil, err
 	}
 
+	sm.Spec.Endpoints[0].TLSConfig.ServerName = fmt.Sprintf("cluster-monitoring-operator.%s.svc", f.namespace)
 	sm.Namespace = f.namespace
 
 	return sm, nil
@@ -2356,6 +2468,19 @@ func (f *Factory) NewAlertmanager(manifest io.Reader) (*monv1.Alertmanager, erro
 	return a, nil
 }
 
+func (f *Factory) NewThanosRuler(manifest io.Reader) (*monv1.ThanosRuler, error) {
+	t, err := NewThanosRuler(manifest)
+	if err != nil {
+		return nil, err
+	}
+
+	if t.GetNamespace() == "" {
+		t.SetNamespace(f.namespaceUserWorkload)
+	}
+
+	return t, nil
+}
+
 func (f *Factory) NewServiceMonitor(manifest io.Reader) (*monv1.ServiceMonitor, error) {
 	sm, err := NewServiceMonitor(manifest)
 	if err != nil {
@@ -2455,6 +2580,7 @@ func (f *Factory) ThanosQuerierDeployment(grpcTLS *v1.Secret, enableUserWorkload
 		d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_THANOS].Args = append(
 			d.Spec.Template.Spec.Containers[THANOS_QUERIER_CONTAINER_THANOS].Args,
 			"--store=dnssrv+_grpc._tcp.prometheus-operated.openshift-user-workload-monitoring.svc.cluster.local",
+			"--store=dnssrv+_grpc._tcp.thanos-ruler-operated.openshift-user-workload-monitoring.svc.cluster.local",
 		)
 	}
 
@@ -2476,12 +2602,28 @@ func (f *Factory) ThanosQuerierDeployment(grpcTLS *v1.Secret, enableUserWorkload
 
 		volume := trustedCABundleVolume(trustedCA.Name, volumeName)
 		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
-			Key:  "ca-bundle.crt",
+			Key:  TrustedCABundleKey,
 			Path: "tls-ca-bundle.pem",
 		})
 		d.Spec.Template.Spec.Volumes = append(d.Spec.Template.Spec.Volumes, volume)
 	}
 
+	if f.config.ThanosQuerierConfig.Resources != nil {
+		for i, c := range d.Spec.Template.Spec.Containers {
+			if c.Name == "thanos-query" {
+				d.Spec.Template.Spec.Containers[i].Resources = *f.config.ThanosQuerierConfig.Resources
+			}
+		}
+	}
+
+	if f.config.ThanosQuerierConfig.NodeSelector != nil {
+		d.Spec.Template.Spec.NodeSelector = f.config.ThanosQuerierConfig.NodeSelector
+	}
+
+	if len(f.config.ThanosQuerierConfig.Tolerations) > 0 {
+		d.Spec.Template.Spec.Tolerations = f.config.ThanosQuerierConfig.Tolerations
+	}
+
 	return d, nil
 }
 
@@ -2632,7 +2774,7 @@ func (f *Factory) TelemeterClientDeployment(proxyCABundleCM *v1.ConfigMap) (*app
 		d.Spec.Template.Spec.Containers[0].VolumeMounts = append(d.Spec.Template.Spec.Containers[0].VolumeMounts, trustedCABundleVolumeMount(volumeName))
 		volume := trustedCABundleVolume(proxyCABundleCM.Name, volumeName)
 		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
-			Key:  "ca-bundle.crt",
+			Key:  TrustedCABundleKey,
 			Path: "tls-ca-bundle.pem",
 		})
 		d.Spec.Template.Spec.Volumes = append(d.Spec.Template.Spec.Volumes, volume)
@@ -2686,6 +2828,201 @@ func (f *Factory) TelemeterClientSecret() (*v1.Secret, error) {
 	return s, nil
 }
 
+func (f *Factory) ThanosRulerService() (*v1.Service, error) {
+	s, err := f.NewService(MustAssetReader(ThanosRulerService))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespaceUserWorkload
+
+	return s, nil
+}
+
+func (f *Factory) ThanosRulerServiceAccount() (*v1.ServiceAccount, error) {
+	s, err := f.NewServiceAccount(MustAssetReader(ThanosRulerServiceAccount))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespaceUserWorkload
+
+	return s, nil
+}
+
+func (f *Factory) ThanosRulerClusterRoleBinding() (*rbacv1.ClusterRoleBinding, error) {
+	crb, err := f.NewClusterRoleBinding(MustAssetReader(ThanosRulerClusterRoleBinding))
+	if err != nil {
+		return nil, err
+	}
+
+	crb.Subjects[0].Namespace = f.namespaceUserWorkload
+
+	return crb, nil
+}
+
+func (f *Factory) ThanosRulerMonitoringClusterRoleBinding() (*rbacv1.ClusterRoleBinding, error) {
+	crb, err := f.NewClusterRoleBinding(MustAssetReader(ThanosRulerMonitoringClusterRoleBinding))
+	if err != nil {
+		return nil, err
+	}
+
+	crb.Subjects[0].Namespace = f.namespaceUserWorkload
+
+	return crb, nil
+}
+
+func (f *Factory) ThanosRulerClusterRole() (*rbacv1.ClusterRole, error) {
+	return f.NewClusterRole(MustAssetReader(ThanosRulerClusterRole))
+}
+
+func (f *Factory) ThanosRulerPrometheusRule() (*monv1.PrometheusRule, error) {
+	return f.NewPrometheusRule(MustAssetReader(ThanosRulerPrometheusRule))
+}
+
+func (f *Factory) ThanosRulerServiceMonitor() (*monv1.ServiceMonitor, error) {
+	sm, err := f.NewServiceMonitor(MustAssetReader(ThanosRulerServiceMonitor))
+	if err != nil {
+		return nil, err
+	}
+
+	sm.Spec.Endpoints[0].TLSConfig.ServerName = fmt.Sprintf("thanos-ruler.%s.svc", f.namespaceUserWorkload)
+	sm.Namespace = f.namespaceUserWorkload
+
+	return sm, nil
+}
+
+func (f *Factory) ThanosRulerRoute() (*routev1.Route, error) {
+	r, err := f.NewRoute(MustAssetReader(ThanosRulerRoute))
+	if err != nil {
+		return nil, err
+	}
+
+	r.Namespace = f.namespaceUserWorkload
+
+	return r, nil
+}
+
+func (f *Factory) ThanosRulerTrustedCABundle() (*v1.ConfigMap, error) {
+	cm, err := f.NewConfigMap(MustAssetReader(ThanosRulerTrustedCABundle))
+	if err != nil {
+		return nil, err
+	}
+
+	return cm, nil
+}
+
+func (f *Factory) ThanosRulerGrpcTLSSecret() (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(ThanosRulerGrpcTLSSecret))
+	if err != nil {
+		return nil, err
+	}
+
+	s.Namespace = f.namespaceUserWorkload
+
+	return s, nil
+}
+
+func (f *Factory) ThanosRulerOauthCookieSecret() (*v1.Secret, error) {
+	s, err := f.NewSecret(MustAssetReader(ThanosRulerOauthCookieSecret))
+	if err != nil {
+		return nil, err
+	}
+
+	p, err := GeneratePassword(43)
+	if err != nil {
+		return nil, err
+	}
+	s.Data["session_secret"] = []byte(p)
+	s.Namespace = f.namespaceUserWorkload
+
+	return s, nil
+}
+
+func (f *Factory) ThanosRulerCustomResource(queryURL string, trustedCA *v1.ConfigMap, grpcTLS *v1.Secret) (*monv1.ThanosRuler, error) {
+	t, err := f.NewThanosRuler(MustAssetReader(ThanosRulerCustomResource))
+	if err != nil {
+		return nil, err
+	}
+
+	t.Spec.Image = f.config.Images.Thanos
+
+	if f.config.ThanosRulerConfig.Resources != nil {
+		t.Spec.Resources = *f.config.ThanosRulerConfig.Resources
+	}
+
+	if f.config.ThanosRulerConfig.VolumeClaimTemplate != nil {
+		t.Spec.Storage = &monv1.StorageSpec{
+			VolumeClaimTemplate: *f.config.ThanosRulerConfig.VolumeClaimTemplate,
+		}
+	}
+
+	if f.config.ThanosRulerConfig.NodeSelector != nil {
+		t.Spec.NodeSelector = f.config.ThanosRulerConfig.NodeSelector
+	}
+
+	if len(f.config.ThanosRulerConfig.Tolerations) > 0 {
+		t.Spec.Tolerations = f.config.ThanosRulerConfig.Tolerations
+	}
+
+	t.Spec.Containers[1].Image = f.config.Images.OauthProxy
+	setEnv := func(name, value string) {
+		for i := range t.Spec.Containers[1].Env {
+			if t.Spec.Containers[1].Env[i].Name == name {
+				t.Spec.Containers[1].Env[i].Value = value
+				break
+			}
+		}
+	}
+	if f.config.HTTPConfig.HTTPProxy != "" {
+		setEnv("HTTP_PROXY", f.config.HTTPConfig.HTTPProxy)
+	}
+	if f.config.HTTPConfig.HTTPSProxy != "" {
+		setEnv("HTTPS_PROXY", f.config.HTTPConfig.HTTPSProxy)
+	}
+	if f.config.HTTPConfig.NoProxy != "" {
+		setEnv("NO_PROXY", f.config.HTTPConfig.NoProxy)
+	}
+
+	// Mounting TLS secret to thanos-ruler
+	if grpcTLS == nil {
+		return nil, errors.New("could not generate thanos ruler CRD: GRPC TLS secret was not found")
+	}
+	secretName := "secret-grpc-tls"
+	secretVolume := v1.Volume{
+		Name: secretName,
+		VolumeSource: v1.VolumeSource{
+			Secret: &v1.SecretVolumeSource{
+				SecretName: grpcTLS.GetName(),
+			},
+		},
+	}
+	t.Spec.Volumes = append(t.Spec.Volumes, secretVolume)
+
+	if trustedCA != nil {
+		volumeName := "thanos-ruler-trusted-ca-bundle"
+		t.Spec.Containers[1].VolumeMounts = append(
+			t.Spec.Containers[1].VolumeMounts,
+			trustedCABundleVolumeMount(volumeName),
+		)
+
+		volume := trustedCABundleVolume(trustedCA.Name, volumeName)
+		volume.VolumeSource.ConfigMap.Items = append(volume.VolumeSource.ConfigMap.Items, v1.KeyToPath{
+			Key:  TrustedCABundleKey,
+			Path: "tls-ca-bundle.pem",
+		})
+		t.Spec.Volumes = append(t.Spec.Volumes, volume)
+	}
+
+	if queryURL != "" {
+		t.Spec.AlertQueryURL = queryURL
+	}
+
+	t.Namespace = f.namespaceUserWorkload
+
+	return t, nil
+}
+
 func NewDaemonSet(manifest io.Reader) (*appsv1.DaemonSet, error) {
 	ds := appsv1.DaemonSet{}
 	err := yaml.NewYAMLOrJSONDecoder(manifest, 100).Decode(&ds)
@@ -2855,6 +3192,16 @@ func NewAlertmanager(manifest io.Reader) (*monv1.Alertmanager, error) {
 	return &a, nil
 }
 
+func NewThanosRuler(manifest io.Reader) (*monv1.ThanosRuler, error) {
+	t := monv1.ThanosRuler{}
+	err := yaml.NewYAMLOrJSONDecoder(manifest, 100).Decode(&t)
+	if err != nil {
+		return nil, err
+	}
+
+	return &t, nil
+}
+
 func NewServiceMonitor(manifest io.Reader) (*monv1.ServiceMonitor, error) {
 	sm := monv1.ServiceMonitor{}
 	err := yaml.NewYAMLOrJSONDecoder(manifest, 100).Decode(&sm)
@@ -2910,25 +3257,29 @@ func NewSecurityContextConstraints(manifest io.Reader) (*securityv1.SecurityCont
 // It adds "monitoring.openshift.io/name" and "monitoring.openshift.io/hash" labels.
 // Any other labels from the given configmap are discarded.
 //
-// It returns nil, if the given configmap does not contain the "ca-bundle.crt" the data key
+// It returns an error if the given configmap does not contain the "ca-bundle.crt" data key
 // or data is empty string.
-func (f *Factory) HashTrustedCA(caBundleCM *v1.ConfigMap, prefix string) *v1.ConfigMap {
-	caBundle, ok := caBundleCM.Data["ca-bundle.crt"]
-	if !ok || caBundle == "" {
-		// We return here instead of erroring out as we need
-		// "ca-bundle.crt" to be there. This can mean that
-		// the CA was not propagated yet. In that case we
-		// will catch this on next sync loop.
-		return nil
+func (f *Factory) HashTrustedCA(caBundleCM *v1.ConfigMap, prefix string) (*v1.ConfigMap, error) {
+	caBundle, ok := caBundleCM.Data[TrustedCABundleKey]
+	if !ok {
+		return nil, errors.Errorf("CA bundle key %q missing", TrustedCABundleKey)
+	}
+	if caBundle == "" {
+		return nil, errors.Errorf("CA bundle key %q empty", TrustedCABundleKey)
 	}
 
 	h := fnv.New64()
 	h.Write([]byte(caBundle))
 	hash := strconv.FormatUint(h.Sum64(), 32)
 
+	ns := f.namespace
+	if caBundleCM.ObjectMeta.Namespace != "" {
+		ns = caBundleCM.ObjectMeta.Namespace
+	}
+
 	return &v1.ConfigMap{
 		ObjectMeta: metav1.ObjectMeta{
-			Namespace: "openshift-monitoring",
+			Namespace: ns,
 			Name:      fmt.Sprintf("%s-trusted-ca-bundle-%s", prefix, hash),
 			Labels: map[string]string{
 				"monitoring.openshift.io/name": prefix,
@@ -2936,9 +3287,9 @@ func (f *Factory) HashTrustedCA(caBundleCM *v1.ConfigMap, prefix string) *v1.Con
 			},
 		},
 		Data: map[string]string{
-			"ca-bundle.crt": caBundle,
+			TrustedCABundleKey: caBundle,
 		},
-	}
+	}, nil
 }
 
 // HashSecret synthesizes a secret by setting the given data
