@@ -25,6 +25,7 @@ import (
 	configv1 "github.com/openshift/api/config/v1"
 
 	v1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 )
 
@@ -134,6 +135,26 @@ func TestUnconfiguredManifests(t *testing.T) {
 		t.Fatal(err)
 	}
 
+	_, err = f.ThanosRulerAlertmanagerConfigSecret()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosRulerQueryConfigSecret()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosRulerMonitoringClusterRoleBinding()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ThanosRulerPrometheusRule()
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	_, err = f.ThanosQuerierGrpcTLSSecret()
 	if err != nil {
 		t.Fatal(err)
@@ -603,6 +624,21 @@ func TestUnconfiguredManifests(t *testing.T) {
 		t.Fatal(err)
 	}
 
+	_, err = f.ClusterMonitoringRulesEditClusterRole()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ClusterMonitoringRulesViewClusterRole()
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	_, err = f.ClusterMonitoringEditClusterRole()
+	if err != nil {
+		t.Fatal(err)
+	}
+
 	_, err = f.ClusterMonitoringOperatorService()
 	if err != nil {
 		t.Fatal(err)
@@ -1135,14 +1171,17 @@ func TestKubeStateMetrics(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	if d.Spec.Template.Spec.Containers[0].Image != "docker.io/openshift/origin-kube-rbac-proxy:latest" {
-		t.Fatal("kube-rbac-proxy image incorrectly configured")
-	}
-	if d.Spec.Template.Spec.Containers[1].Image != "docker.io/openshift/origin-kube-rbac-proxy:latest" {
-		t.Fatal("kube-rbac-proxy image incorrectly configured")
-	}
-	if d.Spec.Template.Spec.Containers[2].Image != "docker.io/openshift/origin-kube-state-metrics:latest" {
-		t.Fatal("kube-state-metrics image incorrectly configured")
+	for i, container := range d.Spec.Template.Spec.Containers {
+		if container.Name == "kube-state-metrics" {
+			if d.Spec.Template.Spec.Containers[i].Image != "docker.io/openshift/origin-kube-state-metrics:latest" {
+				t.Fatal("kube-state-metrics image incorrectly configured")
+			}
+		}
+		if container.Name == "kube-rbac-proxy-self" || container.Name == "kube-rbac-proxy-main" {
+			if d.Spec.Template.Spec.Containers[i].Image != "docker.io/openshift/origin-kube-rbac-proxy:latest" {
+				t.Fatalf("%s image incorrectly configured", container.Name)
+			}
+		}
 	}
 }
 
@@ -1310,3 +1349,94 @@ func TestEtcdGrafanaDashboard(t *testing.T) {
 		t.Fatal("etcd dashboard not found, even if etcd is enabled")
 	}
 }
+
+func TestThanosQuerierConfiguration(t *testing.T) {
+	c, err := NewConfigFromString(`thanosQuerier:
+  nodeSelector:
+    type: foo
+  tolerations:
+  - effect: PreferNoSchedule
+    operator: Exists
+  resources:
+    limits:
+      cpu: 1m
+      memory: 2Mi
+    requests:
+      cpu: 3m
+      memory: 4Mi
+`)
+
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	tls := &v1.Secret{ObjectMeta: metav1.ObjectMeta{Name: "foo"}}
+	f := NewFactory("openshift-monitoring", "openshift-user-workload-monitoring", c)
+	d, err := f.ThanosQuerierDeployment(tls, false, nil)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	for _, tc := range []struct {
+		name      string
+		want, got interface{}
+	}{
+		{
+			name: "node selector",
+			want: map[string]string{"type": "foo"},
+			got:  d.Spec.Template.Spec.NodeSelector,
+		},
+		{
+			name: "tolerations",
+			want: []v1.Toleration{
+				{
+					Effect:   "PreferNoSchedule",
+					Operator: "Exists",
+				},
+			},
+			got: d.Spec.Template.Spec.Tolerations,
+		},
+	} {
+		t.Run(tc.name, func(t *testing.T) {
+			if !reflect.DeepEqual(tc.got, tc.want) {
+				t.Errorf("want %+v, got %+v", tc.want, tc.got)
+			}
+		})
+	}
+
+	for _, c := range d.Spec.Template.Spec.Containers {
+		if c.Name == "thanos-query" {
+			for _, tc := range []struct {
+				name, want string
+				resource   func() *resource.Quantity
+			}{
+				{
+					name:     "limits/cpu",
+					want:     "1m",
+					resource: c.Resources.Limits.Cpu,
+				},
+				{
+					name:     "limits/memory",
+					want:     "2Mi",
+					resource: c.Resources.Limits.Memory,
+				},
+				{
+					name:     "requests/cpu",
+					want:     "3m",
+					resource: c.Resources.Requests.Cpu,
+				},
+				{
+					name:     "requests/memory",
+					want:     "4Mi",
+					resource: c.Resources.Requests.Memory,
+				},
+			} {
+				t.Run(tc.name, func(t *testing.T) {
+					if got := tc.resource(); got.Cmp(resource.MustParse(tc.want)) != 0 {
+						t.Errorf("want %v, got %v", tc.want, got)
+					}
+				})
+			}
+		}
+	}
+}
