@@ -15,8 +15,14 @@
 package framework
 
 import (
+	"bufio"
+	"context"
 	"fmt"
+	"os/exec"
+	"regexp"
+	"strconv"
 	"strings"
+	"testing"
 	"time"
 
 	v1 "k8s.io/api/core/v1"
@@ -31,8 +37,6 @@ import (
 
 	monClient "github.com/coreos/prometheus-operator/pkg/client/versioned/typed/monitoring/v1"
 	"github.com/pkg/errors"
-	apiextensionsclient "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset"
-	crdc "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset/typed/apiextensions/v1beta1"
 	apiservicesclient "k8s.io/kube-aggregator/pkg/client/clientset_generated/clientset"
 
 	metricsclient "k8s.io/metrics/pkg/client/clientset/versioned"
@@ -45,13 +49,13 @@ var (
 
 type Framework struct {
 	OperatorClient      *client.Client
-	CRDClient           crdc.CustomResourceDefinitionInterface
 	KubeClient          kubernetes.Interface
-	ThanosQuerierClient *RouteClient
-	PrometheusK8sClient *RouteClient
-	AlertmanagerClient  *RouteClient
+	ThanosQuerierClient *PrometheusClient
+	PrometheusK8sClient *PrometheusClient
+	AlertmanagerClient  *PrometheusClient
 	APIServicesClient   *apiservicesclient.Clientset
 	MetricsClient       *metricsclient.Clientset
+	kubeConfigPath      string
 
 	MonitoringClient             *monClient.MonitoringV1Client
 	Ns, UserWorkloadMonitoringNs string
@@ -81,12 +85,6 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 		return nil, nil, errors.Wrap(err, "creating monitoring client failed")
 	}
 
-	eclient, err := apiextensionsclient.NewForConfig(config)
-	if err != nil {
-		return nil, nil, errors.Wrap(err, "creating extensions client failed")
-	}
-	crdClient := eclient.ApiextensionsV1beta1().CustomResourceDefinitions()
-
 	operatorClient, err := client.New(config, "", namespaceName, "")
 	if err != nil {
 		return nil, nil, errors.Wrap(err, "creating operator client failed")
@@ -105,12 +103,12 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 	f := &Framework{
 		OperatorClient:           operatorClient,
 		KubeClient:               kubeClient,
-		CRDClient:                crdClient,
 		APIServicesClient:        apiServicesClient,
 		MetricsClient:            metricsClient,
 		MonitoringClient:         mClient,
 		Ns:                       namespaceName,
 		UserWorkloadMonitoringNs: userWorkloadNamespaceName,
+		kubeConfigPath:           kubeConfigPath,
 	}
 
 	cleanUp, err := f.setup()
@@ -118,26 +116,34 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 		return nil, nil, errors.Wrap(err, "failed to setup test framework")
 	}
 
+	token, err := f.GetServiceAccountToken("openshift-monitoring", "cluster-monitoring-operator-e2e")
+	if err != nil {
+		return nil, nil, err
+	}
+
 	// Prometheus client depends on setup above.
-	f.ThanosQuerierClient, err = NewRouteClient(
-		openshiftRouteClient, kubeClient,
+	f.ThanosQuerierClient, err = NewPrometheusClientFromRoute(
+		openshiftRouteClient,
 		"openshift-monitoring", "thanos-querier",
+		token,
 	)
 	if err != nil {
 		return nil, nil, errors.Wrap(err, "creating ThanosQuerierClient failed")
 	}
 
-	f.PrometheusK8sClient, err = NewRouteClient(
-		openshiftRouteClient, kubeClient,
+	f.PrometheusK8sClient, err = NewPrometheusClientFromRoute(
+		openshiftRouteClient,
 		"openshift-monitoring", "prometheus-k8s",
+		token,
 	)
 	if err != nil {
 		return nil, nil, errors.Wrap(err, "creating PrometheusK8sClient failed")
 	}
 
-	f.AlertmanagerClient, err = NewRouteClient(
-		openshiftRouteClient, kubeClient,
+	f.AlertmanagerClient, err = NewPrometheusClientFromRoute(
+		openshiftRouteClient,
 		"openshift-monitoring", "alertmanager-main",
+		token,
 	)
 	if err != nil {
 		return nil, nil, errors.Wrap(err, "creating AlertmanagerClient failed")
@@ -152,14 +158,14 @@ type cleanUpFunc func() error
 func (f *Framework) setup() (cleanUpFunc, error) {
 	cleanUpFuncs := []cleanUpFunc{}
 
-	cf, err := f.CreateServiceAccount()
+	cf, err := f.CreateServiceAccount(f.Ns, "cluster-monitoring-operator-e2e")
 	if err != nil {
 		return nil, err
 	}
 
 	cleanUpFuncs = append(cleanUpFuncs, cf)
 
-	cf, err = f.CreateClusterRoleBinding()
+	cf, err = f.CreateClusterRoleBinding(f.Ns, "cluster-monitoring-operator-e2e", "cluster-monitoring-view")
 	if err != nil {
 		return nil, err
 	}
@@ -187,39 +193,57 @@ func (f *Framework) setup() (cleanUpFunc, error) {
 	}, nil
 }
 
-func (f *Framework) CreateServiceAccount() (cleanUpFunc, error) {
-	serviceAccount := &v1.ServiceAccount{
+func (f *Framework) CreateServiceAccount(namespace, serviceAccount string) (cleanUpFunc, error) {
+	sa := &v1.ServiceAccount{
 		ObjectMeta: metav1.ObjectMeta{
-			Name:      "cluster-monitoring-operator-e2e",
-			Namespace: "openshift-monitoring",
+			Name:      serviceAccount,
+			Namespace: namespace,
 		},
 	}
 
-	serviceAccount, err := f.KubeClient.CoreV1().ServiceAccounts("openshift-monitoring").Create(serviceAccount)
+	sa, err := f.KubeClient.CoreV1().ServiceAccounts(namespace).Create(sa)
 	if err != nil {
 		return nil, err
 	}
 
 	return func() error {
-		return f.KubeClient.CoreV1().ServiceAccounts("openshift-monitoring").Delete(serviceAccount.Name, &metav1.DeleteOptions{})
+		return f.KubeClient.CoreV1().ServiceAccounts(namespace).Delete(sa.Name, &metav1.DeleteOptions{})
 	}, nil
 }
 
-func (f *Framework) CreateClusterRoleBinding() (cleanUpFunc, error) {
+func (f *Framework) GetServiceAccountToken(namespace, name string) (string, error) {
+	secrets, err := f.KubeClient.CoreV1().Secrets(namespace).List(metav1.ListOptions{})
+	if err != nil {
+		return "", err
+	}
+	for _, secret := range secrets.Items {
+		_, dockerToken := secret.Annotations["openshift.io/create-dockercfg-secrets"]
+		token := strings.Contains(secret.Name, fmt.Sprintf("%s-token-", name))
+
+		// we have to skip the token secret that contains the openshift.io/create-dockercfg-secrets annotation
+		// as this is the token to talk to the internal registry.
+		if !dockerToken && token {
+			return string(secret.Data["token"]), nil
+		}
+	}
+	return "", errors.Errorf("cannot find token for %s/%s service account", namespace, name)
+}
+
+func (f *Framework) CreateClusterRoleBinding(namespace, serviceAccount, clusterRole string) (cleanUpFunc, error) {
 	clusterRoleBinding := &rbacv1.ClusterRoleBinding{
 		ObjectMeta: metav1.ObjectMeta{
-			Name: "cluster-monitoring-operator-e2e",
+			Name: fmt.Sprintf("%s-%s", serviceAccount, clusterRole),
 		},
 		Subjects: []rbacv1.Subject{
 			{
 				Kind:      "ServiceAccount",
-				Name:      "cluster-monitoring-operator-e2e",
-				Namespace: "openshift-monitoring",
+				Name:      serviceAccount,
+				Namespace: namespace,
 			},
 		},
 		RoleRef: rbacv1.RoleRef{
 			Kind:     "ClusterRole",
-			Name:     "cluster-monitoring-view",
+			Name:     clusterRole,
 			APIGroup: "rbac.authorization.k8s.io",
 		},
 	}
@@ -234,6 +258,99 @@ func (f *Framework) CreateClusterRoleBinding() (cleanUpFunc, error) {
 	}, nil
 }
 
+func (f *Framework) CreateRoleBindingFromClusterRole(namespace, serviceAccount, clusterRole string) (cleanUpFunc, error) {
+	roleBinding := &rbacv1.RoleBinding{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: fmt.Sprintf("%s-%s", serviceAccount, clusterRole),
+		},
+		Subjects: []rbacv1.Subject{
+			{
+				Kind:      "ServiceAccount",
+				Name:      serviceAccount,
+				Namespace: namespace,
+			},
+		},
+		RoleRef: rbacv1.RoleRef{
+			Kind:     "ClusterRole",
+			Name:     clusterRole,
+			APIGroup: "rbac.authorization.k8s.io",
+		},
+	}
+
+	roleBinding, err := f.KubeClient.RbacV1().RoleBindings(namespace).Create(roleBinding)
+	if err != nil {
+		return nil, err
+	}
+
+	return func() error {
+		return f.KubeClient.RbacV1().RoleBindings(namespace).Delete(roleBinding.Name, &metav1.DeleteOptions{})
+	}, nil
+}
+
+func (f *Framework) ForwardPort(t *testing.T, svc string, port int) (string, func(), error) {
+	t.Helper()
+
+	ctx, cancel := context.WithCancel(context.Background())
+	cmd := exec.CommandContext(ctx, "oc", "port-forward", fmt.Sprintf("service/%s", svc), fmt.Sprintf(":%d", port), "-n", f.Ns, "--config", f.kubeConfigPath)
+
+	cleanUp := func() {
+		cancel()
+		_ = cmd.Wait() // wait to clean up resources but ignore returned error since cancel kills the process
+	}
+
+	stdOut, err := cmd.StdoutPipe()
+	if err != nil {
+		cleanUp()
+		return "", nil, errors.Wrap(err, "fail to open stdout")
+	}
+
+	stdErr, err := cmd.StderrPipe()
+	if err != nil {
+		cleanUp()
+		return "", nil, errors.Wrap(err, "fail to open stderr")
+	}
+	go func() {
+		scanner := bufio.NewScanner(stdErr)
+		for scanner.Scan() {
+			t.Log(scanner.Text())
+		}
+		if err != nil {
+			t.Logf("stderr: %v", scanner.Err())
+		}
+	}()
+
+	err = cmd.Start()
+	if err != nil {
+		cleanUp()
+		return "", nil, errors.Wrap(err, "fail to run command")
+	}
+
+	scanner := bufio.NewScanner(stdOut)
+	if !scanner.Scan() {
+		err := scanner.Err()
+		if err == nil {
+			err = errors.New("got EOF")
+		}
+		cleanUp()
+		return "", nil, errors.Wrap(err, "fail to read stdout")
+	}
+	output := scanner.Text()
+
+	re := regexp.MustCompile(`^Forwarding from [^:]+:(\d+)`)
+	matches := re.FindStringSubmatch(output)
+	if len(matches) != 2 {
+		cleanUp()
+		return "", nil, errors.Wrapf(err, "fail to parse port's value: %q", output)
+	}
+	_, err = strconv.Atoi(matches[1])
+	if err != nil {
+		cleanUp()
+		return "", nil, errors.Wrapf(err, "fail to convert port's value: %q", output)
+	}
+
+	return fmt.Sprintf("127.0.0.1:%s", matches[1]), cleanUp, nil
+}
+
 // Poll calls the given function f every given interval
 // until it returns no error or the given timeout occurs.
 // If a timeout occurs, the last observed error is returned
